class ChartStyle{
  int chartWidth, chartHeight;
  PFont titleFont;
  PFont subtitleFont;
  color chartAreaColor = #F9F9F9;
  color plotAreaColor = #FFFFFF;
  color gridLineColor = #CCCCCC;
  color plotBorderColor= #CCCCCC;
  boolean showTitle = true;
  boolean showSubtitle = false;
  boolean showLegend = false;
  boolean showSource = true;
  boolean horizontal = true;
  int plotX1, plotX2, plotY1, plotY2;
  float barWidth = 0.75;
   
  ChartStyle(String mode){
   if (mode.equals("LARGE")){
      chartWidth = 800;
      chartHeight = 600;
      titleFont = createFont("Gill Sans MT", 48, true);
      subtitleFont = createFont("Gill Sans MT Italic", 32, true);
      //titleFont = loadFont("FrutigerLinotype-Bold-13.vlw");
      //subtitleFont = loadFont("FrutigerLinotype-Roman-12.vlw");
      showSubtitle = true;
      showLegend= true;
  }
    
    if (mode.equals("DEFAULT")){
      chartWidth = 410;
      chartHeight = 320;
      titleFont = createFont("Gill Sans MT", 18, true);
      subtitleFont = createFont("Gill Sans MT Italic", 14, true);
      //titleFont = loadFont("FrutigerLinotype-Bold-13.vlw");
      //subtitleFont = loadFont("FrutigerLinotype-Roman-12.vlw");
      showSubtitle = true;
      showLegend= true;
  }
    if (mode.equals("SMALL")) {
      chartWidth = /*300;*/ 250;
      chartHeight = /*200;*/ 180;
      titleFont=createFont("Gill Sans MT", 10, true);
      subtitleFont = createFont("Verdana", 8, true);
    }
    if (mode.equals("COMITEM")) {
      chartWidth = 130;
      chartHeight = 250;
      titleFont=createFont("Verdana", 7, true);
      subtitleFont = createFont("Verdana", 7, true);
      showSubtitle=false;
      horizontal=false;
    }
      plotX1=5;plotY1=5;
      plotX2=chartWidth-5;plotY2=chartHeight-5;
    }
      
  ChartStyle(int myChartWidth, int myChartHeight, String titleFontName, float titleFontSize, String subtitleFontName, float subtitleFontSize, boolean myShowTitle, boolean myShowSubtitle, boolean myShowLegend, boolean myShowSource, boolean myHorizontal, float myBarWidth,  color myPlotAreaColor, color myChartAreaColor, color myGridLineColor, color myPlotBorderColor, int myPlotX1, int myPlotX2, int myPlotY1, int myPlotY2){
    chartWidth=myChartWidth;
    chartHeight=myChartHeight;
    titleFont=createFont(titleFontName, titleFontSize, true);
    subtitleFont=createFont(subtitleFontName, subtitleFontSize, true);
    showTitle=myShowTitle;
    showSubtitle=myShowSubtitle;
    showLegend=myShowLegend;
    showSource=myShowSource;
    horizontal=myHorizontal;
    barWidth=myBarWidth;
    chartAreaColor = myChartAreaColor;
    plotAreaColor = myPlotAreaColor;
    gridLineColor = myGridLineColor;
    plotBorderColor= myPlotBorderColor;
    plotX1=max(myPlotX1,0);
    plotX2=min(myPlotX2,chartWidth);
    plotY1=max(myPlotY1,0);
    plotY2=min(myPlotY2,chartHeight);
}
  
  void updateSize(Chart myChart){
    boolean debug=false;
    DimAxis myDimAxis = myChart.dimAxis;
    ValAxis myValAxis = myChart.valAxis;
    
    float labelGap = chartWidth*.03;
    float dimGap = chartHeight*.03;
    float titleGap = chartHeight*.02;
    
    float headerSize=0;
    float footerSize=0;
    float leftMargin=0;
    float rightMargin=0;
    if (debug){println("original dimensions:"+plotX1+" "+plotX2+" "+plotY1+" "+plotY2);}
    /*
    1. X-size
    */
        
    textFont(myValAxis.font);
    leftMargin = labelGap+textWidth(nfs(max(abs(myValAxis.maxVal),abs(myValAxis.minVal)),0,0));
    if (myValAxis.hasTitle) {leftMargin+=textAscent()+textDescent();}
    
    rightMargin= labelGap;
    if (myChart.has2Axes){
      rightMargin+=textWidth(nfs(max(abs(myChart.valAxis2.maxVal),abs(myChart.valAxis2.minVal)),0,0));
      if (myChart.valAxis2.hasTitle) {rightMargin+=textAscent()+textDescent();}
    }
    
    if (plotX1<leftMargin){plotX1=int(leftMargin);}
    if (plotX2>myChart.style.chartWidth-rightMargin){plotX2=int(myChart.style.chartWidth-rightMargin);}
    
    if (plotX1<labelGap+textWidth(nfs(max(abs(myValAxis.maxVal),abs(myValAxis.minVal)),0,0))) {
        plotX1 = int(labelGap + textWidth(nfs(max(abs(myValAxis.maxVal),abs(myValAxis.minVal)),0,0)));
    }
    
    /*
     2. Y size
    */
    textFont(titleFont);
    if (showTitle) {headerSize+=textAscent()+textDescent()+chartHeight*.02;}
    textFont(subtitleFont);
    if (showSubtitle) {
      headerSize+=textAscent()+textDescent()/*+max(chartHeight*.02,7)*/;
    }
    if (showLegend){
      headerSize+=textAscent()+textDescent()/*+10*/;}  
    if (plotY1<titleGap+headerSize) {plotY1=int(titleGap+headerSize);}
    
    textFont(myDimAxis.font);
    footerSize+=dimGap+textAscent()+textDescent();
    if (!(myChart.source.equals("")||showSource==false)){
      textFont(subtitleFont);
      footerSize+=textAscent()+textDescent();
    }
    
   if (plotY2>chartHeight-footerSize){plotY2=int(chartHeight-footerSize);}
   
   // extra adjustment if dimension labels written at an angle. This only applies to horizontal type charts.
   // this may affect plotX1 and plotY2.
   
   if(horizontal){
      if (myChart.dimAxis.style.angle!=0) {
        float maxW=0, h=0, s=0;
        float slotWidth; 
        slotWidth = float((plotX2-plotX1))/(myDimAxis.nbValues-(myDimAxis.style.between?0:1));
        textFont(subtitleFont);
        s=(showSource?textAscent()+textDescent():0);
        textFont(myChart.dimAxis.font);
        h=textAscent()+textDescent();
        for(int i=0;i<myDimAxis.nbValues;i+=myDimAxis.style.interval){
          maxW=(textWidth(myDimAxis.labels[i])>maxW)?textWidth(myDimAxis.labels[i]):maxW;
          if (plotX1+(i+(myDimAxis.style.between?.5:0))*slotWidth-cos(myDimAxis.style.angle)*textWidth(myDimAxis.labels[i])-sin(myDimAxis.style.angle)*h<0){
            plotX1=int(-(i+(myDimAxis.style.between?.5:0))*slotWidth+cos(myDimAxis.style.angle)*textWidth(myDimAxis.labels[i])+sin(myDimAxis.style.angle)*h);
          }
        }
        if(plotY2+sin(myDimAxis.style.angle)*maxW+cos(myDimAxis.style.angle)*h+(chartHeight/64)+s>chartHeight) {
          plotY2=int(chartHeight-s-sin(myDimAxis.style.angle)*maxW-cos(myDimAxis.style.angle)*h-(chartHeight/64));
        }
      }
    }
    if (debug) {println("redefining dimensions: "+plotX1+" "+plotX2+" "+plotY1+" "+plotY2);}
  }
  
  void reSize(int newWidth, int newHeight){
      int titleFontSize, subtitleFontSize, paddingSize;
      chartWidth = newWidth;
      chartHeight = newHeight;
      titleFontSize=int(sqrt(chartWidth*chartHeight)/28);
      subtitleFontSize=int(titleFontSize*.7);
      paddingSize=int(chartWidth*chartHeight/25000);
      titleFont=createFont("Verdana bold",  titleFontSize,true);
      subtitleFont = createFont("Verdana", subtitleFontSize,true);
    
      plotX1=plotY1=paddingSize;
      plotX2=chartWidth-paddingSize;plotY2=chartHeight-paddingSize;}
}



class SeriesStyle {
  String type="BAR";
  color seriesFill = #B8CCE4;
  color seriesBorder = #376091;
  
  SeriesStyle(){}
  SeriesStyle(String myType){
    type=myType;}
  SeriesStyle(String myType, color myFill, color myBorder){
    type=myType;
    seriesFill=myFill;
    seriesBorder=myBorder;}
 // for the 2 following constructors, the colors should depend on the series Id.
   SeriesStyle(int seriesId){
     color[] stone = {#1f77b4, #ff7f0e, #2ca02c, #d62728, #9667bd, #8c564b, #e377c2, #7f7f7f, #bcbd22, #17becf};
     seriesFill = stone[seriesId];
     colorMode(HSB);
     seriesBorder = color(hue(seriesFill),saturation(seriesFill),.5*brightness(seriesFill));
     colorMode(RGB);
   }

   SeriesStyle(String myType, int seriesId){
     color[] stone = {#1f77b4, #ff7f0e, #2ca02c, #d62728, #9667bd, #8c564b, #e377c2, #7f7f7f, #bcbd22, #17becf};
     seriesFill = stone[seriesId];
     colorMode(HSB);
     seriesBorder = color(hue(seriesFill),saturation(seriesFill),.5*brightness(seriesFill));
     colorMode(RGB);   
     type=myType;
   }
}

class AxisStyle{
    boolean between = true;
    int interval =1;
    float angle = 0;
    boolean tick = true; 
    AxisStyle(){}
    AxisStyle(boolean myBol){between=myBol;}
    AxisStyle(boolean myBol1, boolean myBol2){between=myBol1;tick=myBol2;}
  }
